@@ -1,9 +1,21 @@
 """Tests for the jwkskate.jwa submodule."""
-
+import pytest
 from binapy import BinaPy
+from cryptography.hazmat.primitives.asymmetric import ec
 
-from jwskate import Jwk
-from jwskate.jwa import Aes128CbcHmacSha256, Aes192CbcHmacSha384, EcdhEs
+from jwskate import (
+    A128CBC_HS256,
+    A128GCM,
+    A192CBC_HS384,
+    A192GCM,
+    A256CBC_HS512,
+    A256GCM,
+    ES256,
+    BaseAESEncryptionAlg,
+    EcdhEs,
+    Jwk,
+    MismatchingAuthTag,
+)
 
 
 def test_aes_128_hmac_sha256() -> None:
@@ -68,7 +80,7 @@ def test_aes_128_hmac_sha256() -> None:
         "65 2c 3f a3 6b 0a 7c 5b 32 19 fa b3 a3 0b c1 c4".replace(" ", "")
     )
 
-    cipher = Aes128CbcHmacSha256(key)
+    cipher = A128CBC_HS256(key)
     assert cipher.aes_key == enc_key
     assert cipher.mac_key == mac_key
     result_ciphertext, result_tag = cipher.encrypt(plaintext, iv=iv, aad=aad)
@@ -146,7 +158,7 @@ def test_aes_192_hmac_sha384() -> None:
         ).replace(" ", "")
     )
 
-    cipher = Aes192CbcHmacSha384(key)
+    cipher = A192CBC_HS384(key)
     assert cipher.aes_key == enc_key
     assert cipher.mac_key == mac_key
     result_ciphertext, result_tag = cipher.encrypt(plaintext, iv=iv, aad=aad)
@@ -191,3 +203,65 @@ def test_ecdhes() -> None:
         key_size=128,
     )
     assert BinaPy(bob_cek).to("b64u") == b"VqqN6vgjbSBcIijNcacQGg"
+
+
+def test_ec_signature_invalid_size() -> None:
+    es256 = ES256(ec.generate_private_key(ec.SECP256R1()).public_key())
+    with pytest.raises(ValueError):
+        es256.verify(b"foo", b"bar")
+
+
+class SupportsBytesTester:
+    """A test class with a __bytes__ method to match SupportBytes interface."""
+
+    def __init__(self, payload: bytes) -> None:
+        self.payload = payload
+
+    def __bytes__(self) -> bytes:  # noqa: D105
+        return self.payload
+
+
+@pytest.mark.parametrize(
+    "alg", [A128GCM, A192GCM, A256GCM, A128CBC_HS256, A192CBC_HS384, A256CBC_HS512]
+)
+def test_encryption(alg: BaseAESEncryptionAlg) -> None:
+    jwa = alg.init_random_key()
+    plaintext = b"this is a test"
+    iv = alg.generate_iv()
+    assert len(iv) * 8 == alg.iv_size
+    ciphertext, tag = jwa.encrypt(plaintext, iv=iv)
+    assert (ciphertext, tag) == jwa.encrypt(
+        SupportsBytesTester(plaintext), iv=SupportsBytesTester(iv)
+    )
+    assert (
+        jwa.decrypt(ciphertext, iv=iv, auth_tag=tag)
+        == jwa.decrypt(
+            SupportsBytesTester(ciphertext),
+            iv=SupportsBytesTester(iv),
+            auth_tag=SupportsBytesTester(tag),
+        )
+        == plaintext
+    )
+
+    with pytest.raises(ValueError):
+        jwa.encrypt(plaintext, iv=b"tooshort")
+    with pytest.raises(ValueError):
+        jwa.encrypt(plaintext, iv=b"toolong" * 50)
+    with pytest.raises(ValueError):
+        jwa.decrypt(ciphertext, iv=b"tooshort", auth_tag=tag)
+    with pytest.raises(ValueError):
+        jwa.decrypt(ciphertext, iv=b"toolong" * 50, auth_tag=tag)
+
+    aad = b"this is an AAD"
+    ciphertext_aad, tag_aad = jwa.encrypt(plaintext, iv=iv, aad=aad)
+    assert (
+        (ciphertext_aad, tag_aad)
+        == jwa.encrypt(plaintext, iv=iv, aad=SupportsBytesTester(aad))
+        != jwa.encrypt(plaintext, iv=iv)
+    )
+    assert jwa.decrypt(ciphertext_aad, auth_tag=tag_aad, iv=iv, aad=aad) == jwa.decrypt(
+        ciphertext_aad, auth_tag=tag_aad, iv=iv, aad=SupportsBytesTester(aad)
+    )
+
+    with pytest.raises(MismatchingAuthTag):
+        jwa.decrypt(ciphertext_aad, auth_tag=tag_aad, iv=iv)
